@@ -5,53 +5,21 @@
 use super::user_data::UserData;
 use crate::{
     assert_size,
+    crypto::{hkdf_rfc_5869, AesGcmResult},
     misc::Flags,
-    request::{
-        hkdf_rfc_5869,
-        openssl::{
-            pkey::{PKey, Private, Public},
-            Md,
-        },
-        uvsecret::{ExtSecret, GuestSecret},
-        Aad, BootHdrTags, Keyslot, ReqEncrCtx, Request, Secret,
-    },
+    req::{Aad, Keyslot, ReqEncrCtx},
+    request::{BootHdrTags, Confidential, Request},
+    secret::{ExtSecret, GuestSecret},
     uv::{ConfigUid, UvFlags},
     Result,
 };
+use openssl::{
+    md::Md,
+    pkey::{PKey, Private, Public},
+};
 use pv_core::request::RequestVersion;
 use zerocopy::AsBytes;
 
-/// Internal wrapper for Guest Secret, so that we can dump it in the form the UV wants it to be
-#[derive(Debug, Clone)]
-struct BinGuestSecret(GuestSecret);
-impl BinGuestSecret {
-    /// Reference to the confidential data
-    fn confidential(&self) -> &[u8] {
-        match &self.0 {
-            GuestSecret::Null => &[],
-            GuestSecret::Association { secret, .. } => secret.value().as_slice(),
-        }
-    }
-    fn dump_auth(&self) -> Vec<u8> {
-        match &self.0 {
-            GuestSecret::Null => vec![0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-            GuestSecret::Association { id, .. } => {
-                let mut buf = vec![0; 48];
-                buf[3] = 2;
-                buf[7] = 0x20;
-                buf[16..48].copy_from_slice(id.as_slice());
-                buf
-            }
-        }
-    }
-}
-
-impl From<GuestSecret> for BinGuestSecret {
-    fn from(secret: GuestSecret) -> Self {
-        BinGuestSecret(secret)
-    }
-}
-
 /// Authenticated data w/o user data
 #[repr(C)]
 #[derive(Debug, Clone, Copy, AsBytes)]
@@ -74,14 +42,14 @@ impl ReqAuthData {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 struct ReqConfData {
-    secret: BinGuestSecret,
-    extension_secret: Secret<[u8; 32]>,
+    secret: GuestSecret,
+    extension_secret: Confidential<[u8; 32]>,
 }
 
 impl ReqConfData {
-    fn to_bytes(&self) -> Secret<Vec<u8>> {
+    fn to_bytes(&self) -> Confidential<Vec<u8>> {
         let secret = self.secret.confidential();
 
         let mut v = vec![0; secret.len() + 32];
@@ -140,10 +108,10 @@ impl From<AddSecretVersion> for RequestVersion {
 
 /// Add-secret request Control Block
 ///
-/// An ASRCB wraps a secret to transport it securely to the Ultravisor.
+/// An ASRCB wraps a secret to securely transport it to the Ultravisor.
 ///
 /// Layout:
-///```none
+/// ```none
 /// _______________________________________________________________
 /// |                   generic header (48)
 /// |     ---------------------------------------------------     |
@@ -162,8 +130,8 @@ impl From<AddSecretVersion> for RequestVersion {
 /// |     ---------------------------------------------------     |
 /// |                   AES GCM Tag (16)                          |
 /// |_____________________________________________________________|
-///```
-#[derive(Clone, Debug)]
+/// ```
+#[derive(Debug)]
 pub struct AddSecretRequest {
     version: AddSecretVersion,
     aad: ReqAuthData,
@@ -180,7 +148,6 @@ impl AddSecretRequest {
     ///
     /// The request has no extension secret, no configuration UID, no host-keys,
     /// and no user data
-    ///
     pub fn new(
         version: AddSecretVersion,
         secret: GuestSecret,
@@ -189,8 +156,8 @@ impl AddSecretRequest {
     ) -> Self {
         AddSecretRequest {
             conf: ReqConfData {
-                extension_secret: Secret::new([0; 32]),
-                secret: secret.into(),
+                extension_secret: Confidential::new([0; 32]),
+                secret,
             },
             aad: ReqAuthData::new(boot_tags, flags),
             keyslots: vec![],
@@ -216,7 +183,7 @@ impl AddSecretRequest {
             ExtSecret::Derived(cck) => hkdf_rfc_5869(
                 Md::sha512(),
                 cck.value(),
-                self.aad.boot_tags.seht(),
+                self.aad.boot_tags.tag(),
                 DER_EXT_SECRET_INFO,
             )?
             .into(),
@@ -226,7 +193,7 @@ impl AddSecretRequest {
 
     /// Returns a reference to the guest secret of this [`AddSecretRequest`].
     pub fn guest_secret(&self) -> &GuestSecret {
-        &self.conf.secret.0
+        &self.conf.secret
     }
 
     /// Add user-data to the Add-Secret request
@@ -239,16 +206,21 @@ impl AddSecretRequest {
     /// - RSA 2048 bit (up to 256 byte message)
     /// - RSA 3072 bit (up to 128 byte message)
     ///
-    /// The signature can be verified during the verification of the secret-request  on the target machine.
-    pub fn set_user_data(&mut self, msg: Vec<u8>, skey: Option<PKey<Private>>) -> Result<()> {
-        self.user_data = UserData::new(skey, msg)?;
+    /// The signature can be verified during the verification of the secret-request  on the target
+    /// machine.
+    pub fn set_user_data<T: Into<Vec<u8>>>(
+        &mut self,
+        msg: T,
+        skey: Option<PKey<Private>>,
+    ) -> Result<()> {
+        self.user_data = UserData::new(skey, msg.into())?;
         Ok(())
     }
 
-    /// compiles the authenticated area of this request
+    /// Compiles the authenticated area of this request
     fn aad(&self, ctx: &ReqEncrCtx, conf_len: usize) -> Result<Vec<u8>> {
         let cust_pub_key = ctx.key_coords()?;
-        let secr_auth = self.conf.secret.dump_auth();
+        let secr_auth = self.conf.secret.auth();
         let user_data = self.user_data.data();
 
         let mut aad: Vec<Aad> = Vec::with_capacity(5 + self.keyslots.len());
@@ -261,7 +233,7 @@ impl AddSecretRequest {
         }
         aad.push(Aad::Plain(cust_pub_key.as_ref()));
         self.keyslots.iter().for_each(|k| aad.push(Aad::Ks(k)));
-        aad.push(Aad::Plain(&secr_auth));
+        aad.push(Aad::Plain(secr_auth.get()));
 
         ctx.build_aad(self.version.into(), &aad, conf_len, self.user_data.magic())
     }
@@ -283,13 +255,18 @@ impl AddSecretRequest {
         Ok(res)
     }
 
-    /// encrypt data, sign request with user-provided signing key, insert signature into aad,
+    /// Encrypts data, sign request with user-provided signing key, insert signature into aad,
     /// calculate request tag
     fn encrypt_with_signed_user_data(&self, ctx: &ReqEncrCtx) -> Result<Vec<u8>> {
-        //encrypt data w/o aead
+        // encrypt data w/o aead
         let conf = self.conf.to_bytes();
         let aad = self.aad(ctx, conf.value().len())?;
-        let (mut buf, aad_range, encr_range, _) = ctx.encrypt_aead(&aad, conf.value())?;
+        let AesGcmResult {
+            mut buf,
+            aad_range,
+            encr_range,
+            ..
+        } = ctx.encrypt_aead(&aad, conf.value())?;
 
         drop(aad);
 
@@ -303,7 +280,7 @@ impl AddSecretRequest {
         // encrypt again with signed data
         buf[encr_range.clone()].copy_from_slice(conf.value());
         ctx.encrypt_aead(&buf[aad_range], &buf[encr_range])
-            .map(|(buf, ..)| buf)
+            .map(|res| res.data())
     }
 }
 
@@ -313,7 +290,7 @@ impl Request for AddSecretRequest {
             UserData::Null | UserData::Unsigned(_) => {
                 let conf = self.conf.to_bytes();
                 let aad = self.aad(ctx, conf.value().len())?;
-                ctx.encrypt_aead(&aad, conf.value()).map(|(buf, ..)| buf)
+                ctx.encrypt_aead(&aad, conf.value()).map(|res| res.data())
             }
             _ => self.encrypt_with_signed_user_data(ctx),
         }
@@ -323,34 +300,3 @@ impl Request for AddSecretRequest {
         self.keyslots.push(Keyslot::new(hostkey))
     }
 }
-
-#[cfg(test)]
-mod test {
-    use super::*;
-
-    #[test]
-    fn guest_secret_bin_null() {
-        let gs: BinGuestSecret = GuestSecret::Null.into();
-        let gs_bytes = gs.dump_auth();
-        let exp = vec![0u8, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
-
-        assert_eq!(exp, gs_bytes);
-        assert_eq!(&Vec::<u8>::new(), gs.confidential())
-    }
-
-    #[test]
-    fn guest_secret_bin_ap() {
-        let gs: BinGuestSecret = GuestSecret::Association {
-            name: "test".to_string(),
-            id: [1; 32],
-            secret: [2; 32].into(),
-        }
-        .into();
-        let gs_bytes_auth = gs.dump_auth();
-        let mut exp = vec![0u8, 0, 0, 2, 0, 0, 0, 0x20, 0, 0, 0, 0, 0, 0, 0, 0];
-        exp.extend([1; 32]);
-
-        assert_eq!(exp, gs_bytes_auth);
-        assert_eq!(&[2; 32], gs.confidential());
-    }
-}
