@@ -15,12 +15,12 @@ export LC_ALL
 ########################################
 # Global used variables
 readonly SCRIPTNAME="${0##*/}" # general name of this script
-readonly STARTDIR="$(pwd)" # save calling directory
+readonly FULLPATHSCRIPT="$(readlink -f "${0}")"
 #
 readonly DATETIME="$(date +%Y-%m-%d-%H-%M-%S 2>/dev/null)"
 readonly DOCKER=$(if type docker >/dev/null 2>&1; then echo "YES"; else echo "NO"; fi)
 readonly DUMP2TAR_OK=$(if type dump2tar >/dev/null 2>&1; then echo "YES"; else echo "NO"; fi)
-readonly HW="$(uname -i 2>/dev/null)"
+readonly HW="$(uname -m 2>/dev/null)"
 readonly IFS_ORI="${IFS}"
 # retrieve and split kernel version
 readonly KERNEL_BASE="$(uname -r 2>/dev/null)"
@@ -1420,9 +1420,8 @@ environment_setup() {
 create_package() {
 	local rc_tar
 	pr_syslog_stdout ${step_num} "Finalizing: Creating archive with collected data"
-	# get a copy of the script used - enabled for relative path calls
-	cd "${STARTDIR}"
-	cp -p "${BASH_SOURCE[0]}" "${WORKPATH}"
+	# get a copy of the script used
+	cp -p "${FULLPATHSCRIPT}" "${WORKPATH}"
 	# create the archive
 	cd "${WORKDIR_BASE}"
 	touch "${WORKARCHIVE}"
