@@ -30,6 +30,14 @@ enum job_id {
 	job_mvdump = 8,
 };
 
+/* target information source */
+typedef enum {
+	source_unknown = 0,
+	source_auto = 1,
+	source_user = 2,
+	source_script = 3
+} source_t;
+
 struct job_target_data {
 	char* bootmap_dir;
 	char* targetbase;
@@ -39,6 +47,7 @@ struct job_target_data {
 	int targetsectors;
 	int targetblocksize;
 	blocknum_t targetoffset;
+	source_t source;
 };
 
 struct job_common_ipl_data {
@@ -122,6 +131,8 @@ struct job_data {
 		struct job_ipl_tape_data ipl_tape;
 		struct job_mvdump_data mvdump;
 	} data;
+	int dump_mounted;
+	int bootmap_dir_created;
 	int noninteractive;
 	int verbose;
 	int add_files;
@@ -131,9 +142,17 @@ struct job_data {
 	int is_ldipl_dump;
 };
 
+/**
+ * Return true, if target parameters for the base disk are set
+ */
+static inline int target_parameters_are_set(struct job_target_data *td)
+{
+	return td->targetbase != NULL;
+}
 
 int job_get(int argc, char* argv[], struct job_data** data);
 void job_free(struct job_data* job);
+void free_target_data(struct job_target_data *td);
 int type_from_target(char *target, disk_type_t *type);
 int check_job_dump_images(struct job_dump_data* dump, char* name);
 int check_job_images_ngdump(struct job_dump_data* dump, char* name);
