@@ -304,7 +304,6 @@ static int add_component_file_range(struct install_set *bis,
 				    address_t load_address,
 				    size_t trailer, void *component,
 				    int add_files,
-				    struct job_target_data *target,
 				    int comp_id, int menu_idx,
 				    int program_table_id)
 {
@@ -339,8 +338,15 @@ static int add_component_file_range(struct install_set *bis,
 			return -1;
 		}
 	} else {
-		/* Make sure file is on correct device */
-		rc = disk_get_info_from_file(filename, target, &file_info);
+		/*
+		 * Make sure that file is on target device.
+		 * For this, retrieve info of the underlying disk without
+		 * any user hints
+		 */
+		struct job_target_data tmp = {.source = source_unknown};
+
+		rc = disk_get_info_from_file(filename, &tmp, &file_info);
+		free_target_data(&tmp);
 		if (rc)
 			return -1;
 		if (file_info->device != bis->info->device) {
@@ -377,12 +383,11 @@ write_segment_table:
 static int add_component_file(struct install_set *bis, const char *filename,
 			      address_t load_address, size_t trailer,
 			      void *component, int add_files,
-			      struct job_target_data *target, int comp_id,
-			      int menu_idx, int program_table_id)
+			      int comp_id, int menu_idx, int program_table_id)
 {
 	return add_component_file_range(bis, filename, NULL, load_address,
 					trailer, component, add_files,
-					target, comp_id, menu_idx,
+					comp_id, menu_idx,
 					program_table_id);
 }
 
@@ -544,8 +549,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 		bool add_envblk, struct job_envblk_data *envblk,
 		struct job_ipl_data *ipl, disk_blockptr_t *program,
 		int verbose, int add_files, component_header_type type,
-		struct job_target_data *target, int is_secure,
-		int menu_idx, int program_table_id)
+		int is_secure, int menu_idx, int program_table_id)
 {
 	struct signature_header sig_head;
 	size_t ramdisk_size, image_size;
@@ -667,7 +671,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 	/* Add stage 3 loader to bootmap */
 	rc = add_component_file(bis, ZIPL_STAGE3_PATH, STAGE3_LOAD_ADDRESS,
 				signature_size, VOID_ADD(table, offset), 1,
-				target, COMPONENT_ID_LOADER, menu_idx,
+				COMPONENT_ID_LOADER, menu_idx,
 				program_table_id);
 	if (rc) {
 		error_text("Could not add internal loader file '%s'",
@@ -745,7 +749,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 
 	rc = add_component_file(bis, ipl->common.image, ipl->common.image_addr,
 				signature_size, VOID_ADD(table, offset),
-				add_files, target, COMPONENT_ID_KERNEL_IMAGE,
+				add_files, COMPONENT_ID_KERNEL_IMAGE,
 				menu_idx, program_table_id);
 	if (rc) {
 		error_text("Could not add image file '%s'", ipl->common.image);
@@ -805,7 +809,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 					ipl->common.ramdisk_addr,
 					signature_size,
 					VOID_ADD(table, offset),
-					add_files, target, COMPONENT_ID_RAMDISK,
+					add_files, COMPONENT_ID_RAMDISK,
 					menu_idx, program_table_id);
 		if (rc) {
 			error_text("Could not add ramdisk '%s'",
@@ -861,7 +865,7 @@ static int add_ipl_program(struct install_set *bis, char *filename,
 			rc = add_component_file_range(bis, filename, &reg,
 						      ipl->envblk_addr, 0,
 						      VOID_ADD(table, offset),
-						      0, target,
+						      0,
 						      COMPONENT_ID_ENVBLK,
 						      menu_idx,
 						      program_table_id);
@@ -893,7 +897,6 @@ static int add_segment_program(struct install_set *bis,
 			       struct job_segment_data *segment,
 			       disk_blockptr_t *program, int verbose,
 			       int add_files, component_header_type type,
-			       struct job_target_data *target,
 			       int program_table_id)
 {
 	void *table;
@@ -913,7 +916,7 @@ static int add_segment_program(struct install_set *bis,
 		printf("  segment file......: %s\n", segment->segment);
 
 	rc = add_component_file(bis, segment->segment, segment->segment_addr, 0,
-				VOID_ADD(table, offset), add_files, target,
+				VOID_ADD(table, offset), add_files,
 				COMPONENT_ID_SEGMENT_FILE, 0 /* menu_idx */,
 				program_table_id);
 	if (rc) {
@@ -996,7 +999,7 @@ static int add_dump_program(struct install_set *bis, struct job_data *job,
 	ipl.common.parmline = dump->common.parmline;
 	ipl.common.parm_addr = dump->common.parm_addr;
 	return add_ipl_program(bis, NULL, false, NULL, &ipl, program,
-			       verbose, 1, type, target, SECURE_BOOT_DISABLED,
+			       verbose, 1, type, SECURE_BOOT_DISABLED,
 			       0 /* menu_idx */, program_table_id);
 }
 
@@ -1041,8 +1044,7 @@ static int build_program_table(struct job_data *job,
 				     true, &job->envblk, &job->data.ipl,
 				     &table[0], verbose || job->command_line,
 				     job->add_files, component_header,
-				     &job->target, job->is_secure, 0,
-				     program_table_id);
+				     job->is_secure, 0, program_table_id);
 		break;
 	case job_segment:
 		if (bis->print_details) {
@@ -1055,7 +1057,7 @@ static int build_program_table(struct job_data *job,
 		rc = add_segment_program(bis, &job->data.segment, &table[0],
 					 verbose || job->command_line,
 					 job->add_files, COMPONENT_HEADER_IPL,
-					 &job->target, program_table_id);
+					 program_table_id);
 		break;
 	case job_dump_partition:
 		/* Only useful for a partition dump that uses a dump kernel*/
@@ -1114,7 +1116,7 @@ static int build_program_table(struct job_data *job,
 					&table[job->data.menu.entry[i].pos],
 					verbose || job->command_line,
 					job->add_files, component_header,
-						     &job->target, is_secure, i,
+						     is_secure, i,
 						     program_table_id);
 				break;
 			case job_print_usage:
@@ -1406,7 +1408,8 @@ check_dump_device(struct job_data *job, const struct disk_info *info,
 	int rc, part_ext;
 
 	/* Check for supported disk and driver types */
-	if ((info->source == source_auto) && (info->type == disk_type_diag)) {
+	if (job->target.source == source_auto &&
+	    info->type == disk_type_diag) {
 		error_reason("Unsupported disk type (%s)",
 			     disk_get_type_name(info->type));
 		return -1;
@@ -1466,6 +1469,7 @@ static int prepare_build_program_table_device(struct job_data *job,
 				   bis->filename);
 			return -1;
 		}
+		bis->tmp_filename_created = 1;
 	}
 	/* Retrieve target device information */
 	if (disk_get_info(bis->filename, &job->target, &bis->info))
@@ -1473,7 +1477,7 @@ static int prepare_build_program_table_device(struct job_data *job,
 
 	if (verbose) {
 		printf("Target device information\n");
-		disk_print_info(bis->info);
+		disk_print_info(bis->info, job->target.source);
 	}
 	if (misc_temp_dev(bis->info->device, 1, &bis->device))
 		return -1;
@@ -1539,14 +1543,14 @@ static int prepare_bootloader_device(struct job_data *job,
  * Prepare resources to build a program table
  */
 static int prepare_build_program_table_file(struct job_data *job,
-					    char *bootmap_dir,
 					    struct install_set *bis)
 {
 	if (bis->skip_prepare)
 		/* skip the preparation work */
 		return 0;
 	/* Create temporary bootmap file */
-	bis->filename = misc_make_path(bootmap_dir, BOOTMAP_TEMPLATE_FILENAME);
+	bis->filename = misc_make_path(job->target.bootmap_dir,
+				       BOOTMAP_TEMPLATE_FILENAME);
 	if (!bis->filename)
 		return -1;
 	bis->fd = mkstemp(bis->filename);
@@ -1555,13 +1559,14 @@ static int prepare_build_program_table_file(struct job_data *job,
 		error_text("Could not create file '%s':", bis->filename);
 		return -1;
 	}
+	bis->tmp_filename_created = 1;
 	/* Retrieve target device information. Note that we have to
 	 * call disk_get_info_from_file() to also get the file system
 	 * block size. */
 	if (disk_get_info_from_file(bis->filename, &job->target, &bis->info))
 		return -1;
 	/* Check for supported disk and driver types */
-	if (bis->info->source == source_auto &&
+	if (job->target.source == source_auto &&
 	    bis->info->type == disk_type_diag) {
 		error_reason("Unsupported disk type (%s)",
 			     disk_get_type_name(bis->info->type));
@@ -1571,7 +1576,7 @@ static int prepare_build_program_table_file(struct job_data *job,
 		return -1;
 	if (verbose) {
 		printf("Target device information\n");
-		disk_print_info(bis->info);
+		disk_print_info(bis->info, job->target.source);
 	}
 	if (misc_temp_dev(bis->info->device, 1, &bis->device))
 		return -1;
@@ -1581,7 +1586,7 @@ static int prepare_build_program_table_file(struct job_data *job,
 					 bis->info))
 			return -1;
 	}
-	printf("Building bootmap in '%s'%s\n", bootmap_dir,
+	printf("Building bootmap in '%s'%s\n", job->target.bootmap_dir,
 	       job->add_files ? " (files will be added to bootmap file)"
 	       : "");
 	/* Initialize bootmap header */
@@ -1601,11 +1606,11 @@ static int prepare_build_program_table_file(struct job_data *job,
 /**
  * Rename to final bootmap name
  */
-static int finalize_create_file(char *bootmap_dir, struct install_set *bis)
+static int finalize_create_file(struct job_data *job, struct install_set *bis)
 {
 	char *final_name;
 
-	final_name = misc_make_path(bootmap_dir, BOOTMAP_FILENAME);
+	final_name = misc_make_path(job->target.bootmap_dir, BOOTMAP_FILENAME);
 	if (!final_name)
 		return -1;
 	if (rename(bis->filename, final_name)) {
@@ -1615,6 +1620,11 @@ static int finalize_create_file(char *bootmap_dir, struct install_set *bis)
 		free(final_name);
 		return -1;
 	}
+	/*
+	 * The temporary object with @bis->filename has been removed
+	 * from the semantic volume
+	 */
+	bis->tmp_filename_created = 0;
 	free(final_name);
 	return 0;
 }
@@ -1623,9 +1633,9 @@ static int finalize_create_file(char *bootmap_dir, struct install_set *bis)
  * PROGRAM_TABLE_ID: offset of the program table in the array (@bis->tables)
  */
 static int bootmap_create_file(struct job_data *job, struct install_set *bis,
-			       char *bootmap_dir, int program_table_id)
+			       int program_table_id)
 {
-	if (prepare_build_program_table_file(job, bootmap_dir, bis))
+	if (prepare_build_program_table_file(job, bis))
 		return -1;
 	if (build_program_table(job, bis, program_table_id))
 		return -1;
@@ -1638,8 +1648,12 @@ static int bootmap_create_file(struct job_data *job, struct install_set *bis,
 	return 0;
 }
 
-static int
-ngdump_create_meta(const char *path)
+/**
+ * Create a file with the short name "ngdump.meta" in the directory PATH.
+ * This file is required for NGDump stand-alone dumper, it's read/written
+ * by the dumper when it starts.
+ */
+static int ngdump_create_meta(const char *path)
 {
 	char *filename = NULL;
 	FILE *fp;
@@ -1676,7 +1690,6 @@ static int prepare_bootloader_ngdump(struct job_data *job,
 				     struct install_set *bis)
 {
 	struct disk_info *info;
-	char *bootmap_dir;
 
 	/* Retrieve target device information */
 	if (disk_get_info(job->data.dump.device, &job->target, &info))
@@ -1686,45 +1699,43 @@ static int prepare_bootloader_ngdump(struct job_data *job,
 	if (check_dump_device(job, info, bis->device))
 		return -1;
 
-	bis->dump_mount_point = misc_make_path("/tmp",
-					       DUMP_TEMP_MOUNT_POINT_NAME);
-	if (!bis->dump_mount_point) {
+	assert(!job->target.bootmap_dir);
+	job->target.bootmap_dir = misc_make_path("/tmp",
+						 DUMP_TEMP_MOUNT_POINT_NAME);
+	if (!job->target.bootmap_dir) {
 		error_reason(strerror(errno));
 		error_text("Could not make path for '%s'",
 			   DUMP_TEMP_MOUNT_POINT_NAME);
 		return -1;
 	}
 	/* Create a mount point directory */
-	if (mkdtemp(bis->dump_mount_point) == NULL) {
+	if (!mkdtemp(job->target.bootmap_dir)) {
 		error_reason(strerror(errno));
 		error_text("Could not create mount point '%s'",
-			   bis->dump_mount_point);
+			   job->target.bootmap_dir);
 		return -1;
 	}
-	bis->dump_tmp_dir_created = 1;
+	job->bootmap_dir_created = 1;
 	/*
 	 * Mount partition where bootmap file and also a dump file will
 	 * be stored.
 	 */
-	if (mount(job->data.dump.device, bis->dump_mount_point,
+	if (mount(job->data.dump.device, job->target.bootmap_dir,
 		  NGDUMP_FSTYPE, 0, NULL)) {
 		error_reason(strerror(errno));
 		error_text("Could not mount partition '%s':",
 			   job->data.dump.device);
 		return -1;
 	}
-	bis->dump_mounted = 1;
-	bootmap_dir = bis->dump_mount_point;
+	job->dump_mounted = 1;
 	/*
 	 * Build a single program table for List-Directed IPL
 	 * See comments before install_bootloader() for details
 	 */
 	bis->print_details = 1;
-	if (bootmap_create_file(job, bis, bootmap_dir, BLKPTR_FORMAT_ID))
+	if (bootmap_create_file(job, bis, BLKPTR_FORMAT_ID))
 		return -1;
-	if (!dry_run && finalize_create_file(bootmap_dir, bis))
-		return -1;
-	return ngdump_create_meta(bootmap_dir);
+	return ngdump_create_meta(job->target.bootmap_dir);
 }
 
 /**
@@ -1734,27 +1745,25 @@ static int prepare_bootloader_ngdump(struct job_data *job,
  */
 static int prepare_bootloader_ipl(struct job_data *job, struct install_set *bis)
 {
-	char *bootmap_dir = job->target.bootmap_dir;
-
 	/*
 	 * Build a program table for List-Directed IPL from
 	 * SCSI or ECKD DASD
 	 */
 	bis->print_details = 1;
-	if (bootmap_create_file(job, bis, bootmap_dir, BLKPTR_FORMAT_ID))
+	if (bootmap_create_file(job, bis, BLKPTR_FORMAT_ID))
 		return -1;
 	if (bis->info->type == disk_type_scsi)
 		/* only one table to be installed per device */
-		return dry_run ? 0 : finalize_create_file(bootmap_dir, bis);
+		return 0;
 	/*
 	 * Build one more program table for CCW-type IPL from
 	 * ECKD DASD
 	 */
 	bis->skip_prepare = 1;
 	bis->print_details = 0;
-	if (bootmap_create_file(job, bis, bootmap_dir, LEGACY_BLKPTR_FORMAT_ID))
+	if (bootmap_create_file(job, bis, LEGACY_BLKPTR_FORMAT_ID))
 		return -1;
-	return dry_run ? 0 : finalize_create_file(bootmap_dir, bis);
+	return 0;
 }
 
 /**
@@ -1800,7 +1809,22 @@ int prepare_bootloader(struct job_data *job, struct install_set *bis)
 	} else {
 		return prepare_bootloader_ipl(job, bis);
 	}
-	return -1;
+}
+
+/**
+ * Do whatever needed after successful boot records installation
+ * but before releasing all the captured resources
+ */
+int post_install_bootloader(struct job_data *job, struct install_set *bis)
+{
+	if (job->id == job_dump_partition) {
+		if (is_ngdump_enabled(job))
+			return dry_run ? 0 : finalize_create_file(job, bis);
+		else
+			return 0;
+	} else {
+		return dry_run ? 0 : finalize_create_file(job, bis);
+	}
 }
 
 /**
@@ -1819,18 +1843,9 @@ void free_bootloader(struct install_set *bis)
 	}
 	if (bis->fd > 0)
 		close(bis->fd);
-	if (dry_run)
+	if (bis->tmp_filename_created)
 		misc_free_temp_file(bis->filename);
 	free(bis->filename);
 	misc_free_temp_dev(bis->device);
 	disk_free_info(bis->info);
-	if (bis->dump_mount_point) {
-		if (bis->dump_mounted && umount(bis->dump_mount_point))
-			warn("Could not umount dump device at %s",
-			     bis->dump_mount_point);
-		if (bis->dump_tmp_dir_created && rmdir(bis->dump_mount_point))
-			warn("Could not remove directory %s",
-			     bis->dump_mount_point);
-		free(bis->dump_mount_point);
-	}
 }
