@@ -18,6 +18,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/mount.h>
 #include <unistd.h>
 #include <assert.h>
 
@@ -369,9 +370,9 @@ get_command_line(int argc, char* argv[], struct command_line* line)
 }
 
 
-static void
-free_target_data(struct job_target_data* data)
+void free_target_data(struct job_target_data *data)
 {
+	free(data->bootmap_dir);
 	free(data->targetbase);
 }
 
@@ -459,7 +460,12 @@ void
 job_free(struct job_data* job)
 {
 
-	free(job->target.bootmap_dir);
+	if (job->dump_mounted && umount(job->target.bootmap_dir))
+		fprintf(stderr, "Could not umount dump device at %s",
+			job->target.bootmap_dir);
+	if (job->bootmap_dir_created && rmdir(job->target.bootmap_dir))
+		fprintf(stderr, "Could not remove directory %s",
+			job->target.bootmap_dir);
 	free_target_data(&job->target);
 	free_envblk_data(&job->envblk);
 	free(job->name);
@@ -1360,6 +1366,7 @@ get_job_from_section_data(char* data[], struct job_data* job, char* section)
 				scan_keyword_targetbase]);
 			if (job->target.targetbase == NULL)
 				return -1;
+			job->target.source = source_user;
 		}
 		if (data[(int) scan_keyword_targettype] != NULL) {
 			if (type_from_target(
@@ -1611,6 +1618,7 @@ get_menu_job(struct scan_token* scan, char* menu, struct job_data* job)
 						scan[i].content.keyword.value);
 					if (job->target.targetbase == NULL)
 						return -1;
+					job->target.source = source_user;
 					break;
 				case scan_keyword_targettype:
 					if (type_from_target(
